#include <SDL/SDL_opengl.h>
#include <SDL/SDL.h>
#include "../headers/functions.h"
#include "../headers/gmath.h"


bool in_triangle(unsigned int terrain_grid[1][9], unsigned int triangle, unsigned int x, unsigned int y)
{	
	return (abs(     
		abs((int)((terrain_grid[triangle][0] - x) * (terrain_grid[triangle][3] - y) - (terrain_grid[triangle][2] - x) * (terrain_grid[triangle][1] - y)))
		+ 
		abs((int)((terrain_grid[triangle][2] - x) * (terrain_grid[triangle][5] - y) - (terrain_grid[triangle][4] - x) * (terrain_grid[triangle][3] - y))) 
		+ 
		abs((int)((terrain_grid[triangle][4] - x) * (terrain_grid[triangle][1] - y) - (terrain_grid[triangle][0] - x) * (terrain_grid[triangle][5] - y))) 
		-	
		abs((int)((terrain_grid[triangle][2] - terrain_grid[triangle][0]) * (terrain_grid[triangle][5] - terrain_grid[triangle][1]) - (terrain_grid[triangle][4] - terrain_grid[triangle][0]) * (terrain_grid[triangle][3] - terrain_grid[triangle][1])))
		) <= (1 / 256));
}

void draw_terrain(unsigned int terrain_grid[][9], unsigned int triangles)
{
    	glDisable(GL_TEXTURE_2D);
    	
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);    			
		glBegin( GL_TRIANGLES );		
		glColor3f(1.0,1.0,1.0);	
		
		// Draw terrain triangles
		for(char i = 1; i < triangles; i++)
		{
			glVertex3f( terrain_grid[i][0], terrain_grid[i][1], 0 );
			glVertex3f( terrain_grid[i][2], terrain_grid[i][3], 0 );
	    	glVertex3f( terrain_grid[i][4], terrain_grid[i][5], 0 );		
		}		    
		glEnd();	
		
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);  			
		glBegin( GL_TRIANGLES );		
		glColor3f(.0,.0,1.0);		
		
		// Draw terrain triangles
		for(char i = 0; i < triangles; i++)
		{
			glVertex3f( terrain_grid[i][0], terrain_grid[i][1], 0 );
			glVertex3f( terrain_grid[i][2], terrain_grid[i][3], 0 );
	    	glVertex3f( terrain_grid[i][4], terrain_grid[i][5], 0 );		
		}		    
		glEnd();			
		
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);		
}

void save_terrain(unsigned int terrain_grid[][9], unsigned int triangles, unsigned int level_w, unsigned int level_h, unsigned char tile_grid[MAX_LEVEL_W/16][MAX_LEVEL_H/16], char tileset, unsigned int objects_grid[][3], unsigned char objects)
{
	FILE* file;
	file = fopen("output","wb");
//file = fopen("output.txt","w");

	//Save room dimensions
	fwrite(&level_w, sizeof(int), 1, file);		
	fwrite(&level_h, sizeof(int), 1, file);			
//fprintf(file,"%d %d\n\n",level_w,level_h);
			
	//Save number of triangles
	fwrite(&triangles, sizeof(int), 1, file);	
//fprintf(file,"%d\n\n",triangles);
		
	//For each triangle:
	for(int i = 1; i < triangles; i++)
	{
		//Save 6 vertices
		fwrite(&terrain_grid[i][0], sizeof(int), 6, file);
//fprintf(file,"%d %d %d %d %d %d\n\n",terrain_grid[i][0],terrain_grid[i][1],terrain_grid[i][2],terrain_grid[i][3],terrain_grid[i][4],terrain_grid[i][5]);

		//Save adjent triangles
		char adjent = 0;
		for(int j = 1; j < triangles; j++)
		{			
			if (i == j)
				continue;
			if (adjent == 3)
				break;
			
			char vertex = 0;
			//if 2 vertices are the same, adjent triangles
			for(char k = 0; k < 3; k++)
				if (terrain_grid[i][0] == terrain_grid[j][k * 2] && terrain_grid[i][1] == terrain_grid[j][k * 2 + 1])
				{
					vertex += 1;
					break;
				}
			for(char k = 0; k < 3; k++)
				if (terrain_grid[i][2] == terrain_grid[j][k * 2] && terrain_grid[i][3] == terrain_grid[j][k * 2 + 1])
				{
					vertex += 1;
					break;
				}
			for(char k = 0; k < 3; k++)
				if (terrain_grid[i][4] == terrain_grid[j][k * 2] && terrain_grid[i][5] == terrain_grid[j][k * 2 + 1])
				{
					vertex += 1;
					break;
				}			
			//Found an adjent, lets write it
			if (vertex == 2) 
			{
				adjent += 1;
				fwrite(&j, sizeof(int), 1, file);													
//fprintf(file,"%d\n\n",j);
			}
		}
		//Add zero's if not enough adjents
		for(char j = adjent; j < 3; j++)
		{
			int var = 0;
			fwrite(&var, sizeof(int), 1, file);	
//fprintf(file,"%d\n\n",var);									
		}
	}	
	//Save tiles
	fwrite(&tileset, sizeof(char), 1, file); //tileset used
	for(unsigned char i = 0; i < level_w/16; i++)
		for(unsigned char j = 0; j < level_h/16; j++)
			fwrite(&tile_grid[i][j], sizeof(char), 1, file);
			
	//Save objects
	for( char i = 0; i < objects; i++ )
	{
		if ( objects_grid[i][0] != 0 ) //not empty slot
		{
			fwrite( &objects_grid[i][0], sizeof(unsigned int), 1, file);
			fwrite( &objects_grid[i][1], sizeof(unsigned int), 1, file);
			fwrite( &objects_grid[i][2], sizeof(unsigned int), 1, file);						
		}
	}
				
	//Close file
	fclose(file);
}

void load_terrain(unsigned int terrain_grid[][9], unsigned int *triangles, unsigned int *level_w, unsigned int *level_h, unsigned char tile_grid[MAX_LEVEL_W/16][MAX_LEVEL_H/16], char * tileset, unsigned int objects_grid[][3])
{
	FILE* file;
	file = fopen("output","rb");
	//Room dimensions
    fread( level_w, sizeof(int), 1, file ); 
    fread( level_h, sizeof(int), 1, file ); 	
	//Amount of triangles
    fread( triangles, sizeof(int), 1, file ); 
    //For every triangle
	for(char i = 1; i < *triangles; i++)
	{
		//Read 9 values (3 vertices -> 6 values) + 3 adjents
	    fread( &terrain_grid[i][0], sizeof(int), 9, file );			    
	}
	fread( tileset, sizeof(char), 1, file );
	//Read tiles
	for(unsigned char i = 0; i < *level_w/16; i++)
		for(unsigned char j = 0; j < *level_h/16; j++)
			fread(&tile_grid[i][j], sizeof(char), 1, file);	
	
	unsigned int temp, readobjects = 0;
	
	fread( &temp, sizeof(unsigned int), 1, file);
	while ( !feof(file) )
	{
		objects_grid[readobjects][0] = temp;
		fread( &objects_grid[readobjects][1], sizeof(unsigned int), 1, file);
		fread( &objects_grid[readobjects][2], sizeof(unsigned int), 1, file);		
		readobjects++;
		fread( &temp, sizeof(unsigned int), 1, file);
	}	
				
	fclose(file);
}

void set_camera( unsigned int level_w, unsigned int level_h, int mouse_x, int mouse_y, unsigned int *camera_x, unsigned int *camera_y)
{
	//Set camera
	//x
	if (mouse_x < 100)
    	*camera_x = (int)max(2, (double)*camera_x - 4, 0);		   		
	else if (mouse_x > 540)
    	*camera_x = (int)min(2, (double)*camera_x + 4.0, (double)level_w - 640.0); 		
	
	if (mouse_y < 100)
    	*camera_y = (int)max(2, (double)*camera_y - 4.0, 0.0);		   		
	else if (mouse_y > 380)
    	*camera_y = (int)min(2, (double)*camera_y + 4.0, (double)level_h - 480.0);
}
