#include <SDL/SDL.h>
#include "../headers/EnemyDogClass.h"
#include "../headers/PlayerClass.h"
#include "../headers/SpriteClass.h"
#include "../headers/ParticleClass.h"
#include "../headers/SoundClass.h"
#include "../headers/gmath.h"
#include "../headers/functions.h"
#include "../headers/defines.h"

extern Sprite mysprite;
extern Sound mysound;
extern Player myplayer;
extern Particle myparticle[100];

EnemyDog::EnemyDog()
{
	x = 0, y = 0, hspeed = 0.0, vspeed = 0.0, w = 32, h = 26, image_index = 0.0, facing = 1, sprite_index = -1, animated = 1, attacking = 0, destroyed = 1;
	current_triangle[0] = current_triangle[1] = current_triangle[2] = current_triangle[3] = -1;
	set_sprite(SPR_DOG_IDLE);	
}

void EnemyDog::create( int _x, int _y )
{
    destroyed = 0, x = _x, y = _y;
}

bool EnemyDog::set_sprite( unsigned char sprite )
{
	if (sprite_index == sprite)
		return 0;
	
	image_index = 0; //only when changing sprite
	
	switch(sprite)
	{
		case SPR_DOG_IDLE:
			sprite_index = sprite;
			images = 4;
			tex_w = 128;
			tex_h = 32;
			img_w = 32;
			img_h = 32;			
			image_speed = 1;
		break;
		case SPR_DOG_RUN:		
			sprite_index = sprite;
			images = 5;
			tex_w = 256;
			tex_h = 32;
			img_w = 35;
			img_h = 32;
			image_speed = 3;
		break;		
	}
	return 1;
}

void EnemyDog::set_x(int _x, unsigned int terrain_grid[][9])
{
    x = _x;
}

void EnemyDog::set_y(int _y, unsigned int terrain_grid[][9])
{
    y = _y;
}

void EnemyDog::set_xy(int _x, int _y, unsigned int terrain_grid[][9])
{
	x = _x;
    y = _y;
}

bool EnemyDog::can_move( unsigned int terrain_grid[][9], int _x, int _y )
{
	int next_triangle[4];
	for ( char i = 0; i <= 3; i++ )
		next_triangle[i] = current_triangle[i];								

	if ( place_free(terrain_grid, x - w/2, y - h/2, _x - w/2, _y - h/2, next_triangle, 0) && place_free(terrain_grid, x + w/2, y - h/2, _x + w/2, _y - h/2, next_triangle, 1) && place_free(terrain_grid, x + w/2, y + h/2, _x + w/2, _y + h/2, next_triangle, 2) && place_free(terrain_grid, x - w/2, y + h/2, _x - w/2, _y + h/2, next_triangle, 3))
	{									
		for ( char i = 0; i <= 3; i++ )
			current_triangle[i] = next_triangle[i];
			
		return 1;
	}
	
	return 0;
}     

void EnemyDog::set_my_triangles( unsigned int terrain_grid[][9], unsigned int triangles )
{
	set_triangles( terrain_grid, current_triangle, x, y, w, h, triangles );
}

void EnemyDog::move(unsigned int terrain_grid[][9], unsigned int triangles)
{	
//	if (current_triangle[0] == -1) //first time
//		set_triangles( terrain_grid, current_triangle, x, y, w, h, triangles );	<
	
	bool see_player = line_free(terrain_grid, x - w/2, y - h/2, myplayer.get_x(), myplayer.get_y(), current_triangle[0]);		
	
	bool move_left = (myplayer.get_x() < x - 46) && see_player;
	bool move_right = (myplayer.get_x() > x + 46) && see_player;	
	
	if (!attacking)
	{
			if ( move_right ) //increase speed
				hspeed = min(2, hspeed + .2, 4.0);
			else if ( !move_left && hspeed > 0 )
				hspeed = max(2, hspeed - .4, 0.0);	
			
			if ( move_left ) //decrease speed
				hspeed = max(2, hspeed - .2, -4.0);
			else if ( !move_right && hspeed < 0 )
				hspeed = min(2, hspeed + .4, 0.0);			
	}

	if (hspeed > 0)
	{	
		facing = 1;
		bool moved = 0;		
		for( char i = 0; i < hspeed; i++)
		{
			for( char j = 0; j < 10; j++ )
			{
				if (can_move(terrain_grid, x + 1, y - j))
				{
					set_xy(x + 1, y - j, terrain_grid);
					moved = 1;
					break;
				}
				hspeed = max(2, 0.0, hspeed - .15); //slopes make EnemyDog go slower
			}
			if (!moved)
				hspeed = 0;							
		}
//		if (moved && (!place_free(terrain_grid, x + w/2, y + h/2 + 4, 2, current_triangle) || !place_free(terrain_grid, x - w/2, y + h/2 + 4, 3, current_triangle)))
	//		if (!mysound.sound_is_playing(SND_EnemyDog_FOOTSTEP))
		//		mysound.play_sound(SND_EnemyDog_FOOTSTEP);		
	}
	else if (hspeed < 0)
	{								
		facing = 0;
		bool moved = 0;		
		for( char i = 0; i > hspeed; i--)
		{
			for( char j = 0; j < 10; j++ )
			{									
				if (can_move(terrain_grid, x - 1, y - j))
				{
					set_xy(x - 1, y - j, terrain_grid);
					moved = 1;
					break;
				}
				hspeed = min(2, 0.0, hspeed + .15); //slopes make EnemyDog go slower
			}
			if (!moved)
				hspeed = 0;	
		}		
//		if (moved && (!place_free(terrain_grid, x + w/2, y + h/2 + 4, 2, current_triangle) || !place_free(terrain_grid, x - w/2, y + h/2 + 4, 3, current_triangle)))
	//		if (!mysound.sound_is_playing(SND_EnemyDog_FOOTSTEP))
		//		mysound.play_sound(SND_EnemyDog_FOOTSTEP);				
	}
	
	if (!attacking)
	{	

		if (vspeed > 0)
		{
			for( char i = 0; i < vspeed; i++)
			{
				if (can_move(terrain_grid, x, y + 1))
					set_y(y + 1, terrain_grid);
				else
				{
					//mysound.sound_set_volume(SND_EnemyDog_LAND, 128 / int(10 / vspeed));
					//mysound.play_sound(SND_EnemyDog_LAND);
					vspeed = 0;
				}
			}
		}
		else
		{	
			for( char i = 0; i > vspeed; i--)
			{
				if (can_move(terrain_grid, x, y - 1))
					set_y(y - 1, terrain_grid);
				else
					vspeed = 0;
			}		
		}	

		//gravity
		if (can_move(terrain_grid, x, y + 1))
			vspeed = min(2, vspeed + 0.4, 6.0);	
		
		vspeed = min(2, vspeed, 12.0);
	}
		
	being_hurt();
		
	update_sprite(terrain_grid);	
}

void EnemyDog::show(/*Uint8 *keystates, */unsigned int terrain_grid[][9])
{	
	//image index controlling
	/*if (sprite_index == SPR_EnemyDog_JUMP && (int)image_index == 3)
	{
		jump(keystates, terrain_grid);		
		animated = 0;
		image_index = 4;
	}*/
	
	//Finish attacking
/*	if (sprite_index == SPR_EnemyDog_AT + weapon && (int)image_index == 11)
		attacking = 0;	*/
	
	//Increase image_index
	if (animated)
		image_index += (double)image_speed / 10.0;

	//Loop animation
	if ((image_index >= images && animated == 1) || images == 1)
		image_index = 0;	
		
	//Draw EnemyDog
	//Body
	Draw_Texture(mysprite.get(sprite_index), x - w/2, y + 4 - h/2, img_w, img_h, tex_w, tex_h, (int)image_index, facing, 0);		
}

void EnemyDog::update_sprite(unsigned int terrain_grid[][9])
{	
	if (hspeed != 0)
		set_sprite(SPR_DOG_RUN);
	else //not moving and on the floor
	{
		//if (attacking)
		//		set_sprite(SPR_EnemyDog_AT);						
			//else
		set_sprite(SPR_DOG_IDLE);			
	}
}

void EnemyDog::set_attacking(bool atk)
{
/*	if (!attacking)
	{
		if (!weapon) //melee
		{
			attacking = (atk && abs(hspeed) <= 0.4 && sprite_index != SPR_EnemyDog_JUMP + weapon);  //only melee ones
			image_index = 0;
			hspeed = .0;
		}
	}*/
}

void EnemyDog::being_hurt()
{
	char amount = 10;
	if (myplayer.get_attacking() && myplayer.get_image_index() == 7 && ((myplayer.get_facing() && x > myplayer.get_x() && x < myplayer.get_x() + 40) || (!myplayer.get_facing() && x < myplayer.get_x()  && x > myplayer.get_x() - 40)))
	{
		for (char i = 0; i < 100 && amount; i++)
		{
			if (myparticle[i].get_destroyed())
			{
				myparticle[i].create(myplayer.get_x() - 25 + 50 * myplayer.get_facing(), y, 100, 0, 0, 1);
				amount -= 1;
			}
		}
	}
}
