#include "../headers/functions.h"
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include "../headers/tga.h"
#include "../headers/gmath.h"
#include "../headers/defines.h"

GLuint LoadTexture(char *TexName)
{
	TGAImg Img;        // Image loader
	GLuint Texture;
 
	// Load our Texture
	if(Img.Load(TexName)!=IMG_OK)
    	return 0;
 
	glGenTextures(1, &Texture);            // Allocate space for texture
	glBindTexture(GL_TEXTURE_2D,Texture); // Set our Tex handle as current
 
	// Create the texture
	if(Img.GetBPP()==24)
    	glTexImage2D(GL_TEXTURE_2D,0,3,Img.GetWidth(),Img.GetHeight(),0,GL_RGB,GL_UNSIGNED_BYTE,Img.GetImg());
    else if(Img.GetBPP()==32)
    	glTexImage2D(GL_TEXTURE_2D,0,4,Img.GetWidth(),Img.GetHeight(),0,GL_RGBA,GL_UNSIGNED_BYTE,Img.GetImg());
    else
    	return 0;
 
   // Specify filtering and edge actions
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
 
   return Texture;
}

void OpenGL_init()
{
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1);
    
	// Set the OpenGL state after creating the context with SDL_SetVideoMode
	glClearColor( 0, 0, 0, 0 );
	
	glEnable (GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	

    glViewport( 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT );

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();

    gluOrtho2D( 0, WINDOW_WIDTH, WINDOW_HEIGHT, 0 ); //left, right, bottom, top
    
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
}
		
void Draw_Texture(unsigned int texture, int x, int y, unsigned char img_w, unsigned char img_h, unsigned int tex_w, unsigned int tex_h, unsigned char img, bool facing, int angle)
{	
	glLoadIdentity();	
	glEnable( GL_TEXTURE_2D ); // Need this to display a texture		
	
	glTranslatef((float)x, (float)y, 0.0f);		
				
	bool mirror_vertical = 0;			
					
	if( angle )
	{			
		if (!facing)
			mirror_vertical = 1;
		facing = 1; //MAY JUST NEED TO MIRROR IT VERTICALLY >: o
		glRotatef(angle,0.0f,0.0f,1.0f);					
	}
	
	// Bind the texture to which subsequent calls refer to
    glBindTexture( GL_TEXTURE_2D, texture );    

	glBegin( GL_QUADS );
	
	// Top-left vertex (corner)
    glTexCoord2d( ((double)((img + 1 - facing) * img_w)) / ((double)tex_w),  ( (double) img_h / (double) tex_h )* ( mirror_vertical )+0 );
	glVertex2f( 0.0, 0.0);
    
	// Bottom-left vertex (corner)
    glTexCoord2d( ((double)((img + facing) * img_w)) / ((double)tex_w), ((double) img_h / (double)tex_h)*(mirror_vertical)+0 );
	glVertex2f( img_w, 0 );
    
    // Bottom-right vertex (corner)
	glTexCoord2d( ((double)((img + facing) * img_w)) / ((double)tex_w), (mirror_vertical) ? 0 : (double) img_h / (double)tex_h );
    glVertex2f( img_w, img_h );
    
	// Top-right vertex (corner)
    glTexCoord2d( ((double)((img + 1 - facing) * img_w)) / ((double)tex_w), (mirror_vertical) ? 0 : (double) img_h / (double)tex_h );
	glVertex2f( 0, img_h );
	
	glEnd();	
	glLoadIdentity();		
}

void draw_tiles(unsigned char tile_grid[MAX_LEVEL_W/16][MAX_LEVEL_H/16], GLuint tex_tileset)
{
	glEnable( GL_TEXTURE_2D );
	glColor3f(1.0,1.0,1.0);				

	for( char i = 0; i < MAX_LEVEL_W/16; i++ )
		for( char j = 0; j < MAX_LEVEL_H/16; j++ )
			Draw_Texture(tex_tileset, i * 16, j * 16, 16, 16, 1024, 16, tile_grid[i][j], 1, 0);
}

void set_camera( int film_x, Sint16 *cam_x, int film_y, Sint16 *cam_y, int level_w, int level_h)
{
	int hborder = WINDOW_WIDTH / 3;
	int vborder = WINDOW_HEIGHT / 3;
	
	if (film_x - *cam_x < hborder)
		*cam_x = (int)max(2, (double)film_x - (double)hborder, 0.0);
	else if (*cam_x + (double)WINDOW_WIDTH - film_x < hborder)
		*cam_x = (int)min(2, (double)film_x + (double)hborder - (double)WINDOW_WIDTH, (double)level_w - (double)WINDOW_WIDTH);		
		
	if (film_y - *cam_y < vborder)
		*cam_y = (int)max(2, (double)film_y - (double)vborder, 0.0);
	else if (*cam_y + (double)WINDOW_HEIGHT - film_y < vborder)
		*cam_y = (int)min(2, (double)film_y + (double)vborder - (double)WINDOW_HEIGHT, (double)level_h - (double)WINDOW_HEIGHT);		
		
	
	glMatrixMode( GL_PROJECTION );
    glLoadIdentity();

    gluOrtho2D( *cam_x, *cam_x + WINDOW_WIDTH, *cam_y + WINDOW_HEIGHT, *cam_y ); //left, right, bottom, top
    
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
}
